#/usr/bin/env python3

import re
import sys

class namelist:
    def __init__(self, filename):
        self.list = {}
        with open(filename, 'r') as f:
            contents = f.read()
        contents = str(re.subn("!.*?\n", "\n", contents))
        blocks = re.finditer('''
            [\$\&]([A-Za-z_]\w*) (?# blockname)
            ([^\$\&\/]*?)        (?# blockbody)
            (?:[\$\&]end|/)      (?# blocktail)
            ''', contents, re.I|re.S|re.X)
        for block in blocks:
            name = block.group(1)
            self.list[name] = {}
            body = block.group(2)
            formulae = re.finditer('''
                ([A-Za-z_]\w*)        (?# LHS)
                [, \t\n;]*=[, \t\n;]* (?# Equal)
                ('([^']|'')*'|"([^"]|"")*"                     (?# string)
                |   \([ \t\n;]*
                       (?:-?\d+|-?\d*\.\d*(?:[de][-+]?\d+)?)
                       [ \t\n;]*,[ \t\n;]*
                       (?:-?\d+|-?\d*\.\d*(?:[de][-+]?\d+)?)
                       [ \t\n;]*\)                             (?# complex)
                |   \.(?:t|f|true|false)\.                     (?# logical)
                |   -?\d*\.\d*(?:[de][-+]?\d+)?                (?# real)
                |   -?\d+                                      (?# integer)
                )''', body, re.I|re.S|re.X)
            for formula in formulae:
                k = formula.group(1)
                v = formula.group(2)
                if re.match("\A'.*'\Z", v, re.I|re.S):
                    v = v.strip("'").replace("''", "'")
                elif re.match('\A".*"\Z', v, re.I|re.S):
                    v = v.strip('"').replace('""', '"')
                elif re.match(r'''
                    \([, \t\n;]*(?:-?\d+|-?\d*\.\d*(?:[de][-+]?\d+)?)
                    [, \t\n;]*,[, \t\n;]*
                    (?:-?\d+|-?\d*\.\d*(?:[de][-+]?\d+)?)[, \t\n;]*\)
                    ''', v, re.I|re.S|re.X):
                    v = v.translate(str.maketrans("", "", ' \t\n;')) \
                         .replace(",", ", ")
                self.list[name][k] = v
    def dump_as_csh(self):
        for group, formulae in self.list.items():
            for k, v in formulae.items():
                sys.stdout.write("set __nml_" + group
                    + "_" + k + " = '" + v + "'\n")

    def __getattr__(self, attrname):
        return getattr(self.list, attrname)
    def __dict__(self): return self.list
    def __str__(self) : return str(self.list)
    def __repr__(self): return repr(self.list)
    def __len__(self) : return len(self.list)
    def __iter__(self): return iter(self.list)
    def __next__(self): return next(self.list)
    def __getitem__(self, key): return self.list[key]

if __name__ == '__main__':
    var = namelist(sys.argv[1])
    var.dump_as_csh()
